@@ -228,8 +228,7 @@ struct symt_module* symt_new_module(struct module* module)
     return sym;
 }
 
-struct symt_compiland* symt_new_compiland(struct module* module, 
-                                          ULONG_PTR address, unsigned src_idx)
+struct symt_compiland* symt_new_compiland(struct module* module, unsigned src_idx)
 {
     struct symt_compiland*    sym;
     struct symt_compiland**   p;
@@ -240,7 +239,7 @@ struct symt_compiland* symt_new_compiland(struct module* module,
     {
         sym->symt.tag  = SymTagCompiland;
         sym->container = module->top;
-        sym->address   = address;
+        sym->address   = 0;
         sym->source    = src_idx;
         vector_init(&sym->vchildren, sizeof(struct symt*), 32);
         sym->user      = NULL;
@@ -305,9 +304,8 @@ struct symt_data* symt_new_global_variable(struct module* module,
         if (type && size && symt_get_info(module, type, TI_GET_LENGTH, &tsz))
         {
             if (tsz != size)
-                FIXME("Size mismatch for %s.%s between type (%s) and src (%Iu)\n",
-                      debugstr_w(module->modulename), name,
-                      wine_dbgstr_longlong(tsz), size);
+                FIXME("Size mismatch for %s.%s between type (%I64u) and src (%Iu)\n",
+                      debugstr_w(module->modulename), name, tsz, size);
         }
         symt_add_module_ht(module, (struct symt_ht*)sym);
         p = vector_add(compiland ? &compiland->vchildren : &module->top->vchildren, &module->pool);
@@ -449,6 +447,7 @@ void symt_add_func_line(struct module* module, struct symt_function* func,
  *
  * Adds a new local/parameter to a given function:
  * In any cases, dt tells whether it's a local variable or a parameter
+ * or a static variable inside the function.
  * If regno it's not 0:
  *      - then variable is stored in a register
  *      - otherwise, value is referenced by register + offset
@@ -470,7 +469,7 @@ struct symt_data* symt_add_func_local(struct module* module,
                          name, type);
 
     assert(symt_check_tag(&func->symt, SymTagFunction) || symt_check_tag(&func->symt, SymTagInlineSite));
-    assert(dt == DataIsParam || dt == DataIsLocal);
+    assert(dt == DataIsParam || dt == DataIsLocal || dt == DataIsStaticLocal);
 
     locsym = pool_alloc(&module->pool, sizeof(*locsym));
     locsym->symt.tag      = SymTagData;
@@ -550,14 +549,13 @@ struct symt_block* symt_open_func_block(struct module* module,
     return block;
 }
 
-struct symt_block* symt_close_func_block(struct module* module, 
+struct symt_block* symt_close_func_block(struct module* module,
                                          const struct symt_function* func,
-                                         struct symt_block* block, unsigned pc)
+                                         struct symt_block* block)
 {
     assert(symt_check_tag(&func->symt, SymTagFunction) || symt_check_tag(&func->symt, SymTagInlineSite));
 
-    if (pc) block->size = func->address + pc - block->address;
-    return (block->container->tag == SymTagBlock) ? 
+    return (block->container->tag == SymTagBlock) ?
         CONTAINING_RECORD(block->container, struct symt_block, symt) : NULL;
 }
 
@@ -804,6 +802,7 @@ static void symt_fill_sym_info(struct module_pair* pair,
                 break;
             case DataIsGlobal:
             case DataIsFileStatic:
+            case DataIsStaticLocal:
                 switch (data->u.var.kind)
                 {
                 case loc_tlsrel:
@@ -887,9 +886,8 @@ static void symt_fill_sym_info(struct module_pair* pair,
     else
         symbol_setname(sym_info, name);
 
-    TRACE_(dbghelp_symt)("%p => %s %lu %s\n",
-                         sym, sym_info->Name, sym_info->Size,
-                         wine_dbgstr_longlong(sym_info->Address));
+    TRACE_(dbghelp_symt)("%p => %s %lu %I64x\n",
+                         sym, sym_info->Name, sym_info->Size, sym_info->Address);
 }
 
 struct sym_enum
@@ -1414,9 +1412,8 @@ BOOL WINAPI SymEnumSymbols(HANDLE hProcess, ULONG64 BaseOfDll, PCSTR Mask,
     BOOL                ret;
     PWSTR               maskW = NULL;
 
-    TRACE("(%p %s %s %p %p)\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), debugstr_a(Mask),
-          EnumSymbolsCallback, UserContext);
+    TRACE("(%p %I64x %s %p %p)\n",
+          hProcess, BaseOfDll, debugstr_a(Mask), EnumSymbolsCallback, UserContext);
 
     if (Mask)
     {
@@ -1651,6 +1648,7 @@ static BOOL find_name(struct process* pcs, struct module* module, const char* na
 BOOL WINAPI SymFromName(HANDLE hProcess, PCSTR Name, PSYMBOL_INFO Symbol)
 {
     struct process*             pcs = process_find_by_handle(hProcess);
+    struct module_pair          pair;
     struct module*              module;
     const char*                 name;
 
@@ -1667,6 +1665,45 @@ BOOL WINAPI SymFromName(HANDLE hProcess, PCSTR Name, PSYMBOL_INFO Symbol)
         module = module_find_by_nameA(pcs, tmp);
         return find_name(pcs, module, name + 1, Symbol);
     }
+
+    /* search first in local context */
+    pair.pcs = pcs;
+    pair.requested = module_find_by_addr(pair.pcs, pcs->localscope_pc, DMT_UNKNOWN);
+    if (module_get_debug(&pair) &&
+        (symt_check_tag(pcs->localscope_symt, SymTagFunction) ||
+         symt_check_tag(pcs->localscope_symt, SymTagInlineSite)))
+    {
+        struct symt_function* func = (struct symt_function*)pcs->localscope_symt;
+        struct vector* v = &func->vchildren;
+        unsigned i;
+
+        for (i = 0; i < vector_length(v); i++)
+        {
+            struct symt* lsym = *(struct symt**)vector_at(v, i);
+            switch (lsym->tag)
+            {
+            case SymTagBlock: /* no recursion */
+                break;
+            case SymTagData:
+                name = symt_get_name(lsym);
+                if (name && !strcmp(name, Name))
+                {
+                    symt_fill_sym_info(&pair, func, lsym, Symbol);
+                    return TRUE;
+                }
+                break;
+            case SymTagLabel: /* not returned here */
+            case SymTagFuncDebugStart:
+            case SymTagFuncDebugEnd:
+            case SymTagCustom:
+            case SymTagInlineSite:
+                break;
+            default:
+                WARN("Unsupported tag: %u (%x)\n", lsym->tag, lsym->tag);
+            }
+        }
+    }
+    /* lookup at global scope */
     for (module = pcs->lmodules; module; module = module->next)
     {
         if (module->type == DMT_PE && find_name(pcs, module, Name, Symbol))
@@ -1684,6 +1721,7 @@ BOOL WINAPI SymFromName(HANDLE hProcess, PCSTR Name, PSYMBOL_INFO Symbol)
                 return TRUE;
         }
     }
+    SetLastError(ERROR_MOD_NOT_FOUND);
     return FALSE;
 }
 
@@ -2448,10 +2486,9 @@ BOOL WINAPI SymSearch(HANDLE hProcess, ULONG64 BaseOfDll, DWORD Index,
     LPWSTR      maskW = NULL;
     BOOLEAN     ret;
 
-    TRACE("(%p %s %lu %lu %s %s %p %p %lx)\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), Index, SymTag, Mask,
-          wine_dbgstr_longlong(Address), EnumSymbolsCallback,
-          UserContext, Options);
+    TRACE("(%p %I64x %lu %lu %s %I64x %p %p %lx)\n",
+          hProcess, BaseOfDll, Index, SymTag, Mask,
+          Address, EnumSymbolsCallback, UserContext, Options);
 
     if (Mask)
     {
@@ -2477,10 +2514,9 @@ BOOL WINAPI SymSearchW(HANDLE hProcess, ULONG64 BaseOfDll, DWORD Index,
 {
     struct sym_enumW    sew;
 
-    TRACE("(%p %s %lu %lu %s %s %p %p %lx)\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), Index, SymTag, debugstr_w(Mask),
-          wine_dbgstr_longlong(Address), EnumSymbolsCallback,
-          UserContext, Options);
+    TRACE("(%p %I64x %lu %lu %s %I64x %p %p %lx)\n",
+          hProcess, BaseOfDll, Index, SymTag, debugstr_w(Mask),
+          Address, EnumSymbolsCallback, UserContext, Options);
 
     sew.ctx = UserContext;
     sew.cb = EnumSymbolsCallback;
@@ -2499,7 +2535,7 @@ BOOL WINAPI SymAddSymbol(HANDLE hProcess, ULONG64 BaseOfDll, PCSTR name,
 {
     struct module_pair  pair;
 
-    TRACE("(%p %s %s %lu)\n", hProcess, wine_dbgstr_a(name), wine_dbgstr_longlong(addr), size);
+    TRACE("(%p %s %I64x %lu)\n", hProcess, wine_dbgstr_a(name), addr, size);
 
     if (!module_init_pair(&pair, hProcess, BaseOfDll)) return FALSE;
 
@@ -2515,7 +2551,7 @@ BOOL WINAPI SymAddSymbolW(HANDLE hProcess, ULONG64 BaseOfDll, PCWSTR nameW,
 {
     char       name[MAX_SYM_NAME];
 
-    TRACE("(%p %s %s %lu)\n", hProcess, wine_dbgstr_w(nameW), wine_dbgstr_longlong(addr), size);
+    TRACE("(%p %s %I64x %lu)\n", hProcess, wine_dbgstr_w(nameW), addr, size);
 
     WideCharToMultiByte(CP_ACP, 0, nameW, -1, name, ARRAY_SIZE(name), NULL, NULL);
 
@@ -2622,8 +2658,8 @@ BOOL WINAPI SymGetLineFromNameW64(HANDLE hProcess, PCWSTR ModuleName, PCWSTR Fil
  */
 BOOL WINAPI SymFromIndex(HANDLE hProcess, ULONG64 BaseOfDll, DWORD index, PSYMBOL_INFO symbol)
 {
-    FIXME("hProcess = %p, BaseOfDll = %s, index = %ld, symbol = %p\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), index, symbol);
+    FIXME("hProcess = %p, BaseOfDll = %I64x, index = %ld, symbol = %p\n",
+          hProcess, BaseOfDll, index, symbol);
 
     return FALSE;
 }
@@ -2634,8 +2670,8 @@ BOOL WINAPI SymFromIndex(HANDLE hProcess, ULONG64 BaseOfDll, DWORD index, PSYMBO
  */
 BOOL WINAPI SymFromIndexW(HANDLE hProcess, ULONG64 BaseOfDll, DWORD index, PSYMBOL_INFOW symbol)
 {
-    FIXME("hProcess = %p, BaseOfDll = %s, index = %ld, symbol = %p\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), index, symbol);
+    FIXME("hProcess = %p, BaseOfDll = %I64x, index = %ld, symbol = %p\n",
+          hProcess, BaseOfDll, index, symbol);
 
     return FALSE;
 }
