@@ -86,6 +86,12 @@ static BOOL console_ioctl( HANDLE handle, DWORD code, void *in_buff, DWORD in_co
     IO_STATUS_BLOCK io;
     NTSTATUS status;
 
+    if (handle == CONSOLE_HANDLE_SHELL_NO_WINDOW)
+    {
+        WARN("Incorrect access to Shell-no-window console (ioctl=%lx)\n", code);
+        SetLastError( ERROR_INVALID_ACCESS );
+        return FALSE;
+    }
     status = NtDeviceIoControlFile( handle, NULL, NULL, NULL, &io, code, in_buff, in_count,
                                     out_buff, out_count );
     switch( status )
@@ -361,10 +367,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH AttachConsole( DWORD pid )
 }
 
 
-/******************************************************************
- *      AllocConsole   (kernelbase.@)
- */
-BOOL WINAPI AllocConsole(void)
+static BOOL alloc_console( BOOL headless )
 {
     SECURITY_ATTRIBUTES inheritable_attr = { sizeof(inheritable_attr), NULL, TRUE };
     STARTUPINFOW app_si, console_si;
@@ -419,6 +422,7 @@ BOOL WINAPI AllocConsole(void)
 
     swprintf( conhost_path, ARRAY_SIZE(conhost_path), L"%s\\conhost.exe", system_dir );
     swprintf( cmd, ARRAY_SIZE(cmd),  L"\"%s\" --server 0x%x", conhost_path, condrv_handle( server ));
+    if (headless) wcscat( cmd, L" --headless" );
     Wow64DisableWow64FsRedirection( &redir );
     ret = CreateProcessW( conhost_path, cmd, NULL, NULL, TRUE, DETACHED_PROCESS, NULL, NULL, &console_si, &pi );
     Wow64RevertWow64FsRedirection( redir );
@@ -444,6 +448,15 @@ error:
 }
 
 
+/******************************************************************
+ *      AllocConsole   (kernelbase.@)
+ */
+BOOL WINAPI AllocConsole(void)
+{
+    return alloc_console( FALSE );
+}
+
+
 /******************************************************************************
  *	CreateConsoleScreenBuffer   (kernelbase.@)
  */
@@ -614,10 +627,13 @@ BOOL WINAPI DECLSPEC_HOTPATCH FreeConsole(void)
 {
     RtlEnterCriticalSection( &console_section );
 
-    NtClose( console_connection );
-    console_connection = NULL;
+    if (RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle != CONSOLE_HANDLE_SHELL_NO_WINDOW)
+    {
+        NtClose( console_connection );
+        console_connection = NULL;
 
-    NtClose( RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle );
+        NtClose( RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle );
+    }
     RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle = NULL;
 
     if (console_flags & CONSOLE_INPUT_HANDLE)  NtClose( GetStdHandle( STD_INPUT_HANDLE ));
@@ -996,12 +1012,11 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetConsoleTitleA( LPSTR title, DWORD size )
     DWORD ret;
 
     if (!ptr) return 0;
+
     ret = GetConsoleTitleW( ptr, size );
     if (ret)
-    {
-        WideCharToMultiByte( GetConsoleOutputCP(), 0, ptr, ret + 1, title, size, NULL, NULL);
-        ret = strlen(title);
-    }
+        WideCharToMultiByte( GetConsoleOutputCP(), 0, ptr, -1, title, size, NULL, NULL);
+
     HeapFree( GetProcessHeap(), 0, ptr );
     return ret;
 }
@@ -1012,15 +1027,26 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetConsoleTitleA( LPSTR title, DWORD size )
  */
 DWORD WINAPI DECLSPEC_HOTPATCH GetConsoleTitleW( LPWSTR title, DWORD size )
 {
-    if (!size) return 0;
+    struct condrv_title_params *params;
+    size_t max_size = sizeof(*params) + (size - 1) * sizeof(WCHAR);
 
-    if (!console_ioctl( RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle, IOCTL_CONDRV_GET_TITLE,
-                        NULL, 0, title, (size - 1) * sizeof(WCHAR), &size ))
+    if (!title || !size) return 0;
+
+    if (!(params = HeapAlloc( GetProcessHeap(), 0, max_size )))
         return 0;
 
-    size /= sizeof(WCHAR);
-    title[size] = 0;
-    return size + 1;
+    if (!console_ioctl( RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle, IOCTL_CONDRV_GET_TITLE,
+                        NULL, 0, params, max_size, &size ))
+        return 0;
+
+    if (size < sizeof(*params)) return 0;
+
+    size -= sizeof(*params);
+    memcpy( title, params->buffer, size );
+    title[ size / sizeof(WCHAR) ] = 0;
+    size = params->title_len;
+    HeapFree( GetProcessHeap(), 0, params );
+    return size;
 }
 
 
@@ -2021,8 +2047,40 @@ BOOL WINAPI ReadConsoleW( HANDLE handle, void *buffer, DWORD length, DWORD *coun
         return FALSE;
     }
 
-    ret = console_ioctl( handle, IOCTL_CONDRV_READ_CONSOLE, NULL, 0, buffer,
-                         length * sizeof(WCHAR), count );
+    if (reserved)
+    {
+        CONSOLE_READCONSOLE_CONTROL* crc = reserved;
+        char *tmp;
+
+        if (crc->nLength != sizeof(*crc) || crc->nInitialChars >= length)
+        {
+            SetLastError( ERROR_INVALID_PARAMETER );
+            return FALSE;
+        }
+        if (!(tmp = HeapAlloc( GetProcessHeap(), 0, sizeof(DWORD) + length * sizeof(WCHAR) )))
+        {
+            SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+            return FALSE;
+        }
+
+        memcpy( tmp, &crc->dwCtrlWakeupMask, sizeof(DWORD) );
+        memcpy( tmp + sizeof(DWORD), buffer, crc->nInitialChars * sizeof(WCHAR) );
+        ret = console_ioctl( handle, IOCTL_CONDRV_READ_CONSOLE_CONTROL,
+                             tmp, sizeof(DWORD) + crc->nInitialChars * sizeof(WCHAR),
+                             tmp, sizeof(DWORD) + length * sizeof(WCHAR), count );
+        if (ret)
+        {
+            memcpy( &crc->dwConsoleKeyState, tmp, sizeof(DWORD) );
+            *count -= sizeof(DWORD);
+            memcpy( buffer, tmp + sizeof(DWORD), *count );
+        }
+        HeapFree( GetProcessHeap(), 0, tmp );
+    }
+    else
+    {
+        ret = console_ioctl( handle, IOCTL_CONDRV_READ_CONSOLE, NULL, 0, buffer,
+                             length * sizeof(WCHAR), count );
+    }
     if (ret) *count /= sizeof(WCHAR);
     return ret;
 }
@@ -2255,12 +2313,15 @@ void init_console( void )
             init_console_std_handles( FALSE );
         }
     }
-    else if (params->ConsoleHandle == CONSOLE_HANDLE_ALLOC)
+    else if (params->ConsoleHandle == CONSOLE_HANDLE_ALLOC ||
+             params->ConsoleHandle == CONSOLE_HANDLE_ALLOC_NO_WINDOW)
     {
+        BOOL no_window = params->ConsoleHandle == CONSOLE_HANDLE_ALLOC_NO_WINDOW;
         HMODULE mod = GetModuleHandleW( NULL );
         params->ConsoleHandle = NULL;
         if (RtlImageNtHeader( mod )->OptionalHeader.Subsystem == IMAGE_SUBSYSTEM_WINDOWS_CUI)
-            AllocConsole();
+            alloc_console( no_window );
     }
-    else if (params->ConsoleHandle) create_console_connection( params->ConsoleHandle );
+    else if (params->ConsoleHandle && params->ConsoleHandle != CONSOLE_HANDLE_SHELL_NO_WINDOW)
+        create_console_connection( params->ConsoleHandle );
 }
