@@ -316,6 +316,7 @@ sync_test("prevent_default", function() {
     a.addEventListener("click", function(e) {
         calls += "a,";
         ok(e.defaultPrevented === true, "e.defaultPrevented = " + e.defaultPrevented);
+        ok(e.isTrusted === true, "isTrusted = " + e.isTrusted);
     }, true);
 
     calls = "";
@@ -386,12 +387,14 @@ sync_test("init_event", function() {
     ok(e.type === "", "type = " + e.type);
     ok(e.cancelable === false, "cancelable = " + e.cancelable);
     ok(e.bubbles === false, "bubbles = " + e.bubbles);
+    ok(e.isTrusted === false, "isTrusted = " + e.isTrusted);
 
     e.initEvent("test", true, false);
     ok(e.type === "test", "type = " + e.type);
     ok(e.cancelable === false, "cancelable = " + e.cancelable);
     ok(e.bubbles === true, "bubbles = " + e.bubbles);
     ok(e.defaultPrevented === false, "defaultPrevented = " + e.defaultPrevented);
+    ok(e.isTrusted === false, "isTrusted = " + e.isTrusted);
 
     e.preventDefault();
     ok(e.defaultPrevented === false, "defaultPrevented = " + e.defaultPrevented);
@@ -406,6 +409,7 @@ sync_test("init_event", function() {
 
     elem.addEventListener("NewTest", function(event) {
         ok(e === event, "e != event");
+        ok(e.isTrusted === false, "isTrusted = " + e.isTrusted);
 
         e.preventDefault();
         ok(e.defaultPrevented === true, "defaultPrevented = " + e.defaultPrevented);
@@ -792,3 +796,28 @@ async_test("detached_img_error_event", function() {
     }
     img.src = "about:blank";
 });
+
+async_test("img_wrong_content_type", function() {
+    var img = new Image();
+    img.onload = function() {
+        ok(img.width === 2, "width = " + img.width);
+        ok(img.height === 2, "height = " + img.height);
+        next_test();
+    }
+    img.src = "img.png?content-type=image/jpeg";
+});
+
+async_test("message event", function() {
+    var listener_called = false;
+
+    window.addEventListener("message", function(e) {
+        listener_called = true;
+        ok(e.data === "test", "e.data = " + e.data);
+        ok(e.bubbles === false, "bubbles = " + e.bubbles);
+        ok(e.cancelable === false, "cancelable = " + e.cancelable);
+        next_test();
+    });
+
+    window.postMessage("test", "httP://wineTest.example.org");
+    ok(listener_called == false, "listener already called");
+});
