@@ -32,11 +32,9 @@
 #include "commctrl.h"
 #include "objbase.h"
 #include "docobj.h"
-#include "undocshell.h"
 #include "shlobj.h"
 #include "shellapi.h"
 #include "wine/heap.h"
-#include "wine/unicode.h"
 #include "wine/list.h"
 
 /*******************************************
@@ -104,6 +102,8 @@ HRESULT WINAPI QueryAssociations_Constructor(IUnknown *pUnkOuter, REFIID riid, L
 HRESULT WINAPI ExplorerBrowser_Constructor(IUnknown *pUnkOuter, REFIID riid, LPVOID *ppv) DECLSPEC_HIDDEN;
 HRESULT WINAPI KnownFolderManager_Constructor(IUnknown *pUnkOuter, REFIID riid, LPVOID *ppv) DECLSPEC_HIDDEN;
 HRESULT WINAPI IFileOperation_Constructor(IUnknown *outer, REFIID riid, void **out) DECLSPEC_HIDDEN;
+HRESULT WINAPI ActiveDesktop_Constructor(IUnknown *outer, REFIID riid, void **out) DECLSPEC_HIDDEN;
+
 extern HRESULT CPanel_GetIconLocationW(LPCITEMIDLIST, LPWSTR, UINT, int*) DECLSPEC_HIDDEN;
 HRESULT WINAPI CPanel_ExtractIconA(LPITEMIDLIST pidl, LPCSTR pszFile, UINT nIconIndex, HICON *phiconLarge, HICON *phiconSmall, UINT nIconSize) DECLSPEC_HIDDEN;
 HRESULT WINAPI CPanel_ExtractIconW(LPITEMIDLIST pidl, LPCWSTR pszFile, UINT nIconIndex, HICON *phiconLarge, HICON *phiconSmall, UINT nIconSize) DECLSPEC_HIDDEN;
@@ -112,6 +112,7 @@ HRESULT WINAPI IAutoComplete_Constructor(IUnknown * pUnkOuter, REFIID riid, LPVO
 HRESULT WINAPI ApplicationAssociationRegistration_Constructor(IUnknown *outer, REFIID riid, LPVOID *ppv) DECLSPEC_HIDDEN;
 
 HRESULT WINAPI ApplicationDestinations_Constructor(IUnknown *outer, REFIID riid, LPVOID *ppv) DECLSPEC_HIDDEN;
+HRESULT WINAPI ApplicationDocumentLists_Constructor(IUnknown *outer, REFIID riid, LPVOID *ppv) DECLSPEC_HIDDEN;
 
 HRESULT IShellLink_ConstructFromFile(IUnknown * pUnkOuter, REFIID riid, LPCITEMIDLIST pidl, IUnknown **ppv) DECLSPEC_HIDDEN;
 
@@ -177,7 +178,6 @@ static inline WCHAR * __SHCloneStrAtoW(WCHAR ** target, const char * source)
 
 extern WCHAR swShell32Name[MAX_PATH] DECLSPEC_HIDDEN;
 
-BOOL UNIXFS_is_rooted_at_desktop(void) DECLSPEC_HIDDEN;
 extern const GUID CLSID_UnixFolder DECLSPEC_HIDDEN;
 extern const GUID CLSID_UnixDosFolder DECLSPEC_HIDDEN;
 
@@ -269,4 +269,29 @@ static inline WCHAR *strdupAtoW(const char *str)
     return ret;
 }
 
+/* explorer ("cabinet") window messages */
+#define CWM_SETPATH             (WM_USER + 2)
+#define CWM_WANTIDLE            (WM_USER + 3)
+#define CWM_GETSETCURRENTINFO   (WM_USER + 4)
+#define CWM_SELECTITEM          (WM_USER + 5)
+#define CWM_SELECTITEMSTR       (WM_USER + 6)
+#define CWM_GETISHELLBROWSER    (WM_USER + 7)
+#define CWM_TESTPATH            (WM_USER + 9)
+#define CWM_STATECHANGE         (WM_USER + 10)
+#define CWM_GETPATH             (WM_USER + 12)
+
+/* CWM_TESTPATH types */
+#define CWTP_ISEQUAL  0
+#define CWTP_ISCHILD  1
+
+/* CWM_TESTPATH structure */
+typedef struct
+{
+    DWORD dwType;
+    ITEMIDLIST idl;
+} CWTESTPATHSTRUCT;
+
+BOOL WINAPI StrRetToStrNA(char *, DWORD, STRRET *, const ITEMIDLIST *);
+BOOL WINAPI StrRetToStrNW(WCHAR *, DWORD, STRRET *, const ITEMIDLIST *);
+
 #endif
