@@ -19,8 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,8 +39,6 @@
 #include "winreg.h"
 #include "winerror.h"
 
-#include "undocshell.h"
-#include "wine/unicode.h"
 #include "shell32_main.h"
 
 #include "wine/debug.h"
@@ -68,6 +64,7 @@ static const struct {
 
 	{&CLSID_ApplicationAssociationRegistration, ApplicationAssociationRegistration_Constructor},
 	{&CLSID_ApplicationDestinations, ApplicationDestinations_Constructor},
+	{&CLSID_ApplicationDocumentLists, ApplicationDocumentLists_Constructor},
 	{&CLSID_AutoComplete,   IAutoComplete_Constructor},
 	{&CLSID_ControlPanel,	IControlPanel_Constructor},
 	{&CLSID_DragDropHelper, IDropTargetHelper_Constructor},
@@ -90,6 +87,7 @@ static const struct {
 	{&CLSID_DestinationList, CustomDestinationList_Constructor},
 	{&CLSID_ShellImageDataFactory, ShellImageDataFactory_Constructor},
 	{&CLSID_FileOperation, IFileOperation_Constructor},
+	{&CLSID_ActiveDesktop, ActiveDesktop_Constructor},
 	{NULL, NULL}
 };
 
@@ -120,10 +118,7 @@ HRESULT WINAPI SHCoCreateInstance(
 	IID	iid;
 	const	CLSID * myclsid = clsid;
 	WCHAR	sKeyName[MAX_PATH];
-	static const WCHAR sCLSID[] = {'C','L','S','I','D','\\','\0'};
 	WCHAR	sClassID[60];
-	static const WCHAR sInProcServer32[] = {'\\','I','n','p','r','o','c','S','e','r','v','e','r','3','2','\0'};
-	static const WCHAR sLoadWithoutCOM[] = {'L','o','a','d','W','i','t','h','o','u','t','C','O','M','\0'};
 	WCHAR	sDllPath[MAX_PATH];
 	HKEY	hKey = 0;
 	DWORD	dwSize;
@@ -152,15 +147,13 @@ HRESULT WINAPI SHCoCreateInstance(
 
 	/* we look up the dll path in the registry */
         SHStringFromGUIDW(myclsid, sClassID, ARRAY_SIZE(sClassID));
-	lstrcpyW(sKeyName, sCLSID);
-	lstrcatW(sKeyName, sClassID);
-	lstrcatW(sKeyName, sInProcServer32);
+        swprintf( sKeyName, ARRAY_SIZE(sKeyName), L"CLSID\\%s\\InprocServer32", sClassID );
 
 	if (RegOpenKeyExW(HKEY_CLASSES_ROOT, sKeyName, 0, KEY_READ, &hKey))
             return E_ACCESSDENIED;
 
         /* if a special registry key is set, we load a shell extension without help of OLE32 */
-        if (!SHQueryValueExW(hKey, sLoadWithoutCOM, 0, 0, 0, 0))
+        if (!SHQueryValueExW(hKey, L"LoadWithoutCOM", 0, 0, 0, 0))
         {
 	    /* load an external dll without ole32 */
 	    HANDLE hLibrary;
@@ -180,7 +173,7 @@ HRESULT WINAPI SHCoCreateInstance(
 		hres = E_ACCESSDENIED;
 	        goto end;
             } else if (FAILED(hres = DllGetClassObject(myclsid, &IID_IClassFactory, (LPVOID*)&pcf))) {
-		    TRACE("GetClassObject failed 0x%08x\n", hres);
+		    TRACE("GetClassObject failed 0x%08lx\n", hres);
 		    goto end;
 	    }
 
@@ -196,7 +189,7 @@ end:
         if (hKey) RegCloseKey(hKey);
 	if(hres!=S_OK)
 	{
-	  ERR("failed (0x%08x) to create CLSID:%s IID:%s\n",
+	  ERR("failed (0x%08lx) to create CLSID:%s IID:%s\n",
               hres, shdebugstr_guid(myclsid), shdebugstr_guid(refiid));
 	  ERR("class not found in registry\n");
 	}
@@ -316,7 +309,7 @@ LPVOID WINAPI SHAlloc(DWORD len)
 	LPVOID ret;
 
 	ret = CoTaskMemAlloc(len);
-	TRACE("%u bytes at %p\n",len, ret);
+	TRACE("%lu bytes at %p\n",len, ret);
 	return ret;
 }
 
@@ -353,7 +346,7 @@ HRESULT WINAPI SHGetDesktopFolder(IShellFolder **psf)
 	*psf = NULL;
 	hres = ISF_Desktop_Constructor(NULL, &IID_IShellFolder, (LPVOID*)psf);
 
-	TRACE("-- %p->(%p) 0x%08x\n", psf, *psf, hres);
+	TRACE("-- %p->(%p) 0x%08lx\n", psf, *psf, hres);
 	return hres;
 }
 /**************************************************************************
@@ -433,7 +426,7 @@ static ULONG WINAPI IDefClF_fnAddRef(LPCLASSFACTORY iface)
 	IDefClFImpl *This = impl_from_IClassFactory(iface);
 	ULONG refCount = InterlockedIncrement(&This->ref);
 
-	TRACE("(%p)->(count=%u)\n", This, refCount - 1);
+	TRACE("(%p)->(count=%lu)\n", This, refCount - 1);
 
 	return refCount;
 }
@@ -445,7 +438,7 @@ static ULONG WINAPI IDefClF_fnRelease(LPCLASSFACTORY iface)
 	IDefClFImpl *This = impl_from_IClassFactory(iface);
 	ULONG refCount = InterlockedDecrement(&This->ref);
 
-	TRACE("(%p)->(count=%u)\n", This, refCount + 1);
+	TRACE("(%p)->(count=%lu)\n", This, refCount + 1);
 
 	if (!refCount)
 	{
@@ -667,7 +660,7 @@ UINT WINAPI DragQueryFileW(
 	  }
 	}
 
-	i = strlenW(lpwDrop);
+	i = lstrlenW(lpwDrop);
 	if ( !lpszwFile) goto end;   /* needed buffer size */
 	lstrcpynW (lpszwFile, lpwDrop, lLength);
 end:
@@ -686,7 +679,7 @@ HRESULT WINAPI SHPropStgCreate(IPropertySetStorage *psstg, REFFMTID fmtid,
     PROPVARIANT ret;
     HRESULT hres;
 
-    TRACE("%p %s %s %x %x %x %p %p\n", psstg, debugstr_guid(fmtid), debugstr_guid(pclsid),
+    TRACE("%p %s %s %lx %lx %lx %p %p\n", psstg, debugstr_guid(fmtid), debugstr_guid(pclsid),
             grfFlags, grfMode, dwDisposition, ppstg, puCodePage);
 
     hres = IPropertySetStorage_Open(psstg, fmtid, grfMode, ppstg);
@@ -718,7 +711,7 @@ HRESULT WINAPI SHPropStgCreate(IPropertySetStorage *psstg, REFFMTID fmtid,
             if(FAILED(hres) || ret.vt!=VT_I2)
                 *puCodePage = 0;
             else
-                *puCodePage = ret.u.iVal;
+                *puCodePage = ret.iVal;
         }
     }
 
@@ -734,7 +727,7 @@ HRESULT WINAPI SHPropStgReadMultiple(IPropertyStorage *pps, UINT uCodePage,
     STATPROPSETSTG stat;
     HRESULT hres;
 
-    FIXME("%p %u %u %p %p\n", pps, uCodePage, cpspec, rgpspec, rgvar);
+    FIXME("%p %u %lu %p %p\n", pps, uCodePage, cpspec, rgpspec, rgvar);
 
     memset(rgvar, 0, cpspec*sizeof(PROPVARIANT));
     hres = IPropertyStorage_ReadMultiple(pps, cpspec, rgpspec, rgvar);
@@ -751,7 +744,7 @@ HRESULT WINAPI SHPropStgReadMultiple(IPropertyStorage *pps, UINT uCodePage,
         if(FAILED(hres) || ret.vt!=VT_I2)
             return S_OK;
 
-        uCodePage = ret.u.iVal;
+        uCodePage = ret.iVal;
     }
 
     hres = IPropertyStorage_Stat(pps, &stat);
@@ -772,7 +765,7 @@ HRESULT WINAPI SHPropStgWriteMultiple(IPropertyStorage *pps, UINT *uCodePage,
     UINT codepage;
     HRESULT hres;
 
-    FIXME("%p %p %u %p %p %d\n", pps, uCodePage, cpspec, rgpspec, rgvar, propidNameFirst);
+    FIXME("%p %p %lu %p %p %ld\n", pps, uCodePage, cpspec, rgpspec, rgvar, propidNameFirst);
 
     hres = IPropertyStorage_Stat(pps, &stat);
     if(FAILED(hres))
@@ -789,10 +782,10 @@ HRESULT WINAPI SHPropStgWriteMultiple(IPropertyStorage *pps, UINT *uCodePage,
         hres = IPropertyStorage_ReadMultiple(pps, 1, &prop, &ret);
         if(FAILED(hres))
             return hres;
-        if(ret.vt!=VT_I2 || !ret.u.iVal)
+        if(ret.vt!=VT_I2 || !ret.iVal)
             return E_FAIL;
 
-        codepage = ret.u.iVal;
+        codepage = ret.iVal;
         if(uCodePage)
             *uCodePage = codepage;
     }
@@ -868,7 +861,7 @@ static ULONG WINAPI ShellImageData_AddRef(IShellImageData *iface)
     ShellImageData *This = impl_from_IShellImageData(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("%p, %u\n", This, ref);
+    TRACE("%p, %lu\n", This, ref);
 
     return ref;
 }
@@ -878,7 +871,7 @@ static ULONG WINAPI ShellImageData_Release(IShellImageData *iface)
     ShellImageData *This = impl_from_IShellImageData(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("%p, %u\n", This, ref);
+    TRACE("%p, %lu\n", This, ref);
 
     if (!ref)
     {
@@ -896,7 +889,7 @@ static HRESULT WINAPI ShellImageData_Decode(IShellImageData *iface, DWORD flags,
     GpImage *image;
     HRESULT hr;
 
-    TRACE("%p, %#x, %u, %u\n", This, flags, cx_desired, cy_desired);
+    TRACE("%p, %#lx, %lu, %lu\n", This, flags, cx_desired, cy_desired);
 
     if (This->image)
         return S_FALSE;
@@ -1060,7 +1053,7 @@ static HRESULT WINAPI ShellImageDate_SelectPage(IShellImageData *iface, ULONG pa
 {
     ShellImageData *This = impl_from_IShellImageData(iface);
 
-    FIXME("%p, %u: stub\n", This, page);
+    FIXME("%p, %lu: stub\n", This, page);
 
     return E_NOTIMPL;
 }
@@ -1117,7 +1110,7 @@ static HRESULT WINAPI ShellImageData_GetProperties(IShellImageData *iface, DWORD
 {
     ShellImageData *This = impl_from_IShellImageData(iface);
 
-    FIXME("%p, %#x, %p: stub\n", This, mode, props);
+    FIXME("%p, %#lx, %p: stub\n", This, mode, props);
 
     return E_NOTIMPL;
 }
@@ -1126,7 +1119,7 @@ static HRESULT WINAPI ShellImageData_Rotate(IShellImageData *iface, DWORD angle)
 {
     ShellImageData *This = impl_from_IShellImageData(iface);
 
-    FIXME("%p, %u: stub\n", This, angle);
+    FIXME("%p, %lu: stub\n", This, angle);
 
     return E_NOTIMPL;
 }
@@ -1135,7 +1128,7 @@ static HRESULT WINAPI ShellImageData_Scale(IShellImageData *iface, ULONG cx, ULO
 {
     ShellImageData *This = impl_from_IShellImageData(iface);
 
-    FIXME("%p, %u, %u, %#x: stub\n", This, cx, cy, mode);
+    FIXME("%p, %lu, %lu, %#x: stub\n", This, cx, cy, mode);
 
     return E_NOTIMPL;
 }
