@@ -21,6 +21,7 @@
 
 #include "wine/debug.h"
 #include "wine/heap.h"
+#include "wine/rbtree.h"
 #include "initguid.h"
 #include "wine/iaccessible2.h"
 
@@ -1130,6 +1131,8 @@ HRESULT WINAPI UiaProviderFromIAccessible(IAccessible *acc, long child_id, DWORD
  */
 struct uia_provider_thread
 {
+    struct rb_tree node_map;
+    struct list nodes_list;
     HANDLE hthread;
     HWND hwnd;
     LONG ref;
@@ -1145,6 +1148,138 @@ static CRITICAL_SECTION_DEBUG provider_thread_cs_debug =
 };
 static CRITICAL_SECTION provider_thread_cs = { &provider_thread_cs_debug, -1, 0, 0, 0, 0 };
 
+struct uia_provider_thread_map_entry
+{
+    struct rb_entry entry;
+
+    SAFEARRAY *runtime_id;
+    struct list nodes_list;
+};
+
+static int uia_runtime_id_compare(const void *key, const struct rb_entry *entry)
+{
+    struct uia_provider_thread_map_entry *prov_entry = RB_ENTRY_VALUE(entry, struct uia_provider_thread_map_entry, entry);
+    return uia_compare_runtime_ids(prov_entry->runtime_id, (SAFEARRAY *)key);
+}
+
+void uia_provider_thread_remove_node(HUIANODE node)
+{
+    struct uia_node *node_data = impl_from_IWineUiaNode((IWineUiaNode *)node);
+
+    TRACE("Removing node %p\n", node);
+
+    EnterCriticalSection(&provider_thread_cs);
+
+    list_remove(&node_data->prov_thread_list_entry);
+    list_init(&node_data->prov_thread_list_entry);
+    if (!list_empty(&node_data->node_map_list_entry))
+    {
+        list_remove(&node_data->node_map_list_entry);
+        list_init(&node_data->node_map_list_entry);
+        if (list_empty(&node_data->map->nodes_list))
+        {
+            rb_remove(&provider_thread.node_map, &node_data->map->entry);
+            SafeArrayDestroy(node_data->map->runtime_id);
+            heap_free(node_data->map);
+        }
+        node_data->map = NULL;
+    }
+
+    LeaveCriticalSection(&provider_thread_cs);
+}
+
+static void uia_provider_thread_disconnect_node(SAFEARRAY *sa)
+{
+    struct rb_entry *rb_entry;
+
+    EnterCriticalSection(&provider_thread_cs);
+
+    /* Provider thread hasn't been started, no nodes to disconnect. */
+    if (!provider_thread.ref)
+        goto exit;
+
+    rb_entry = rb_get(&provider_thread.node_map, sa);
+    if (rb_entry)
+    {
+        struct uia_provider_thread_map_entry *prov_map;
+        struct list *cursor, *cursor2;
+        struct uia_node *node_data;
+
+        prov_map = RB_ENTRY_VALUE(rb_entry, struct uia_provider_thread_map_entry, entry);
+        LIST_FOR_EACH_SAFE(cursor, cursor2, &prov_map->nodes_list)
+        {
+            node_data = LIST_ENTRY(cursor, struct uia_node, node_map_list_entry);
+
+            list_remove(cursor);
+            list_remove(&node_data->prov_thread_list_entry);
+            list_init(&node_data->prov_thread_list_entry);
+            list_init(&node_data->node_map_list_entry);
+            node_data->map = NULL;
+
+            IWineUiaNode_disconnect(&node_data->IWineUiaNode_iface);
+        }
+
+        rb_remove(&provider_thread.node_map, &prov_map->entry);
+        SafeArrayDestroy(prov_map->runtime_id);
+        heap_free(prov_map);
+    }
+
+exit:
+    LeaveCriticalSection(&provider_thread_cs);
+}
+
+static HRESULT uia_provider_thread_add_node(HUIANODE node)
+{
+    struct uia_node *node_data = impl_from_IWineUiaNode((IWineUiaNode *)node);
+    struct uia_provider *prov_data = impl_from_IWineUiaProvider(node_data->prov);
+    SAFEARRAY *sa;
+    HRESULT hr;
+
+    node_data->nested_node = prov_data->return_nested_node = TRUE;
+    hr = UiaGetRuntimeId(node, &sa);
+    if (FAILED(hr))
+        return hr;
+
+    TRACE("Adding node %p\n", node);
+
+    EnterCriticalSection(&provider_thread_cs);
+    list_add_tail(&provider_thread.nodes_list, &node_data->prov_thread_list_entry);
+
+    /* If we have a runtime ID, create an entry in the rb tree. */
+    if (sa)
+    {
+        struct uia_provider_thread_map_entry *prov_map;
+        struct rb_entry *rb_entry;
+
+        if ((rb_entry = rb_get(&provider_thread.node_map, sa)))
+        {
+            prov_map = RB_ENTRY_VALUE(rb_entry, struct uia_provider_thread_map_entry, entry);
+            SafeArrayDestroy(sa);
+        }
+        else
+        {
+            prov_map = heap_alloc_zero(sizeof(*prov_map));
+            if (!prov_map)
+            {
+                SafeArrayDestroy(sa);
+                LeaveCriticalSection(&provider_thread_cs);
+                return E_OUTOFMEMORY;
+            }
+
+            prov_map->runtime_id = sa;
+            list_init(&prov_map->nodes_list);
+            rb_put(&provider_thread.node_map, sa, &prov_map->entry);
+        }
+
+        list_add_tail(&prov_map->nodes_list, &node_data->node_map_list_entry);
+        node_data->map = prov_map;
+    }
+
+    LeaveCriticalSection(&provider_thread_cs);
+
+    return S_OK;
+}
+
 #define WM_GET_OBJECT_UIA_NODE (WM_USER + 1)
 #define WM_UIA_PROVIDER_THREAD_STOP (WM_USER + 2)
 static LRESULT CALLBACK uia_provider_thread_msg_proc(HWND hwnd, UINT msg, WPARAM wparam,
@@ -1155,11 +1290,14 @@ static LRESULT CALLBACK uia_provider_thread_msg_proc(HWND hwnd, UINT msg, WPARAM
     case WM_GET_OBJECT_UIA_NODE:
     {
         HUIANODE node = (HUIANODE)lparam;
-        struct uia_node *node_data;
         LRESULT lr;
 
-        node_data = impl_from_IWineUiaNode((IWineUiaNode *)node);
-        node_data->nested_node = TRUE;
+        if (FAILED(uia_provider_thread_add_node(node)))
+        {
+            WARN("Failed to add node %p to provider thread list.\n", node);
+            UiaNodeRelease(node);
+            return 0;
+        }
 
         /*
          * LresultFromObject returns an index into the global atom string table,
@@ -1239,6 +1377,8 @@ static BOOL uia_start_provider_thread(void)
         GetModuleHandleExW(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
                 (const WCHAR *)uia_start_provider_thread, &hmodule);
 
+        list_init(&provider_thread.nodes_list);
+        rb_init(&provider_thread.node_map, uia_runtime_id_compare);
         events[0] = ready_event = CreateEventW(NULL, FALSE, FALSE, NULL);
         if (!(provider_thread.hthread = CreateThread(NULL, 0, uia_provider_thread_proc,
                 ready_event, 0, NULL)))
@@ -1276,6 +1416,8 @@ void uia_stop_provider_thread(void)
     {
         PostMessageW(provider_thread.hwnd, WM_UIA_PROVIDER_THREAD_STOP, 0, 0);
         CloseHandle(provider_thread.hthread);
+        if (!list_empty(&provider_thread.nodes_list))
+            ERR("Provider thread shutdown with nodes still in the list\n");
         memset(&provider_thread, 0, sizeof(provider_thread));
     }
     LeaveCriticalSection(&provider_thread_cs);
@@ -1286,7 +1428,7 @@ void uia_stop_provider_thread(void)
  * Automation has to work regardless of whether or not COM is initialized on
  * the thread calling UiaReturnRawElementProvider.
  */
-static LRESULT uia_lresult_from_node(HUIANODE huianode)
+LRESULT uia_lresult_from_node(HUIANODE huianode)
 {
     if (!uia_start_provider_thread())
     {
@@ -1329,3 +1471,33 @@ LRESULT WINAPI UiaReturnRawElementProvider(HWND hwnd, WPARAM wparam,
 
     return uia_lresult_from_node(node);
 }
+
+/***********************************************************************
+ *          UiaDisconnectProvider (uiautomationcore.@)
+ */
+HRESULT WINAPI UiaDisconnectProvider(IRawElementProviderSimple *elprov)
+{
+    SAFEARRAY *sa;
+    HUIANODE node;
+    HRESULT hr;
+
+    TRACE("(%p)\n", elprov);
+
+    hr = UiaNodeFromProvider(elprov, &node);
+    if (FAILED(hr))
+        return hr;
+
+    hr = UiaGetRuntimeId(node, &sa);
+    UiaNodeRelease(node);
+    if (FAILED(hr))
+        return hr;
+
+    if (!sa)
+        return E_INVALIDARG;
+
+    uia_provider_thread_disconnect_node(sa);
+
+    SafeArrayDestroy(sa);
+
+    return S_OK;
+}
