@@ -196,7 +196,7 @@ static UINT set_palette_entries( HPALETTE hpalette, UINT start, UINT count,
 
     TRACE("hpal=%p,start=%i,count=%i\n",hpalette,start,count );
 
-    if (hpalette == get_stock_object(DEFAULT_PALETTE)) return 0;
+    if (hpalette == GetStockObject(DEFAULT_PALETTE)) return 0;
     palPtr = GDI_GetObjPtr( hpalette, NTGDI_OBJ_PAL );
     if (!palPtr) return 0;
 
@@ -249,7 +249,7 @@ static BOOL animate_palette( HPALETTE hPal, UINT StartIndex, UINT NumEntries,
 {
     TRACE("%p (%i - %i)\n", hPal, StartIndex,StartIndex+NumEntries);
 
-    if( hPal != get_stock_object(DEFAULT_PALETTE) )
+    if( hPal != GetStockObject(DEFAULT_PALETTE) )
     {
         PALETTEOBJ * palPtr;
         UINT pal_entries;
@@ -405,7 +405,7 @@ COLORREF CDECL nulldrv_GetNearestColor( PHYSDEV dev, COLORREF color )
         PALETTEENTRY entry;
         HPALETTE hpal = dc->hPalette;
 
-        if (!hpal) hpal = get_stock_object( DEFAULT_PALETTE );
+        if (!hpal) hpal = GetStockObject( DEFAULT_PALETTE );
         if (spec_type == 2) /* PALETTERGB */
             index = NtGdiGetNearestPaletteIndex( hpal, color );
         else  /* PALETTEINDEX */
@@ -514,14 +514,14 @@ HPALETTE WINAPI NtUserSelectPalette( HDC hdc, HPALETTE hpal, WORD bkg )
 
     TRACE("%p %p\n", hdc, hpal );
 
-    if (!bkg && hpal != get_stock_object( DEFAULT_PALETTE ))
+    if (!bkg && hpal != GetStockObject( DEFAULT_PALETTE ))
     {
-        HWND hwnd = user_callbacks->pWindowFromDC( hdc );
+        HWND hwnd = NtUserWindowFromDC( hdc );
         if (hwnd)
         {
             /* set primary palette if it's related to current active */
             HWND foreground = NtUserGetForegroundWindow();
-            is_primary = foreground == hwnd || user_callbacks->pIsChild( foreground, hwnd );
+            is_primary = foreground == hwnd || is_child( foreground, hwnd );
         }
     }
 
@@ -555,7 +555,7 @@ UINT realize_palette( HDC hdc )
 
     /* FIXME: move primary palette handling from user32 */
 
-    if( dc->hPalette == get_stock_object( DEFAULT_PALETTE ))
+    if( dc->hPalette == GetStockObject( DEFAULT_PALETTE ))
     {
         PHYSDEV physdev = GET_DC_PHYSDEV( dc, pRealizeDefaultPalette );
         realized = physdev->funcs->pRealizeDefaultPalette( physdev );
@@ -582,18 +582,14 @@ UINT realize_palette( HDC hdc )
     if (realized && is_primary)
     {
         /* send palette change notification */
-        HWND hwnd = user_callbacks->pWindowFromDC( hdc );
-        if (hwnd) user_callbacks->pSendMessageTimeoutW( HWND_BROADCAST, WM_PALETTECHANGED,
-                                                        HandleToUlong(hwnd), 0, SMTO_ABORTIFHUNG,
-                                                        2000, NULL );
+        HWND hwnd = NtUserWindowFromDC( hdc );
+        if (hwnd) send_message_timeout( HWND_BROADCAST, WM_PALETTECHANGED, HandleToUlong(hwnd), 0,
+                                        SMTO_ABORTIFHUNG, 2000, FALSE );
     }
     return realized;
 }
 
 
-typedef HWND (WINAPI *WindowFromDC_funcptr)( HDC );
-typedef BOOL (WINAPI *RedrawWindow_funcptr)( HWND, const RECT *, HRGN, UINT );
-
 /**********************************************************************
  *           NtGdiUpdateColors    (win32u.@)
  *
@@ -605,14 +601,13 @@ BOOL WINAPI NtGdiUpdateColors( HDC hDC )
     HWND hwnd;
 
     if (!size) return FALSE;
-    if (!user_callbacks) return TRUE;
 
-    hwnd = user_callbacks->pWindowFromDC( hDC );
+    hwnd = NtUserWindowFromDC( hDC );
 
     /* Docs say that we have to remap current drawable pixel by pixel
      * but it would take forever given the speed of XGet/PutPixel.
      */
-    if (hwnd && size) user_callbacks->pRedrawWindow( hwnd, NULL, 0, RDW_INVALIDATE );
+    if (hwnd && size) NtUserRedrawWindow( hwnd, NULL, 0, RDW_INVALIDATE );
     return TRUE;
 }
 
